# Copyright (C) 2004 Neil Stevens <neil@hakubi.us>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
# AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# Except as contained in this notice, the name(s) of the author(s) shall not be
# used in advertising or otherwise to promote the sale, use or other dealings
# in this Software without prior written authorization from the author(s).

module Flexo
	def Flexo.alternateNames(name)
		retval = []

		gnomeName = name.gsub(/([a-z])([A-Z])/) do
			$1 + '_' + $2.downcase
		end
		if gnomeName != name
			retval << gnomeName
		end

		return retval
	end
end

class Object
	alias :flexoMethodMissing :method_missing

	def method_missing(id, *args, &block)
		# no infinite loops
		if caller.first.include?('flexo.rb')
			return flexoMethodMissing(id, *args, &block)
		end

		name = id.id2name
		Flexo::alternateNames(name).each do |alternate|
			if respond_to?(alternate)
				return method(alternate).call(*args, &block)
			end
		end
		return flexoMethodMissing(id, *args, &block)
	end
end
